<?php
namespace Teknisa\PLA\Modules\Transportadas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO\Response\Error;

class Pla04100ServicosTransportados {
	protected $connection;
	protected $environment;
	protected $util;
	protected $crudService;

	public function __construct($connection, $environment, $util, $crudService) {
		$this->connection    = $connection;
		$this->environment   = $environment;
		$this->util          = $util;
		$this->crudService = $crudService;
	}
    
    public function returnComensais(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();

    	$sql = "SELECT SUM(QTCOMESTICAR) QTCOMESTICAR 
    	          FROM CARDADIA 
    			 WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI 
    			   AND DTCARDAPIO = :P_DTCARDAPIO";
	
		$params = array(
			"P_CDFILIAL"   => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_DTCARDAPIO" => $row["DTSERVTRAN"]
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
		return $result;
    }
    
    public function calculate(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();
    	$nrorg = $this->environment->getNrOrg();
    	$cdoperador = $this->environment->getCdOperador();
    	$concat = '';
		$filiServ = '';
		$filiServConcat = [];
    	if ($row['TODOS_SERV'] == 'N') {
    		foreach($row['NMSERVICO_CALC'] as $serv)  {
	            $concat .= $row['UNIDADE'].$serv.'_'; 
	        }
	        $filiServ = explode("_", $concat);
	        array_pop($filiServ);
    	} else {
			$sql = "SELECT SUBSTR(S.CDFILIAL || S.CDSERVFILI, 1, 9) AS FILISERV 
			          FROM SERVICO S, FILIOPER F 
			         WHERE S.CDFILIAL = :CDFILIAL 
			           AND (F.CDOPERADOR = :CDOPERADOR) 
			           AND (F.CDFILIAL = S.CDFILIAL) AND (S.NRORG = :NRORG) 
			         ORDER BY NLSSORT(S.CDFILIAL,'NLS_SORT=BINARY_AI') , NLSSORT(S.CDSERVFILI,'NLS_SORT=BINARY_AI')";
			
			$params = array(
				"CDOPERADOR" => $cdoperador,
				"NRORG"      => $nrorg,
				"CDFILIAL"   => $row['UNIDADE']
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
	    	$result = $stmt->fetchAll();
			foreach($result as $unServ) {
		        array_push($filiServConcat, $unServ['FILISERV']);
			}
		    $filiServ = $filiServConcat;
	    }

	    $nrlistaserv = $this->util->insertLista($filiServ);
	    $this->procedureServTransp($nrlistaserv, $row, $cdoperador);
    }
    
    public function calculateNotFoundRegister(DTO\Request\Row $request, DTO\Response $response) {
    	$row = $request->getRow();
    	$nrorg = $this->environment->getNrOrg();
    	$cdoperador = $this->environment->getCdOperador();
    	
    	$row['UNIDADE'] = $row['CDFILIAL'];
    	$row['DTSERVTRAN_CALC'] = $row['DTSERVTRAN'];
    	
    	$concat = $row['CDFILIAL'].$row['CDSERVFILI']; 
    	$filiServ = explode("_", $concat);
    	
	    $nrlistaserv = $this->util->insertLista($filiServ);
	    $this->procedureServTransp($nrlistaserv, $row, $cdoperador);
    }
    
    public function procedureServTransp($nrlistaserv, $row, $cdoperador) {
    	$today = date("d/m/Y G:i:s");

    	$procedure = new StoredProcedure($this->connection, 'SERVICOS_TRANSPORTADOS');
    	
    	$procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $row['UNIDADE'],         Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_SERVICOS',   Param::PARAM_INPUT, $nrlistaserv,            Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $row['DTSERVTRAN_CALC'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $row['DTSERVTRAN_CALC'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador,             Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTHRCALCST', Param::PARAM_INPUT, $today,                  Param::PARAM_TYPE_STR));
    	$procedure->execute();
    	$this->util->deleteLista($nrlistaserv);
    }
    
    private function returnPlates($rows) {
    	$sqlPratos = "SELECT ST.NRSERVTRAN, ST.CDFILIAL, ST.CDSERVFILI, ST.DTSERVTRAN, ST.CDPRATO, ST.CDFILAUXPRAT, 
    						 ST.CDDIETPADST, PR.NMPRATO, NVL(PR.VRARREDTRANS, '0') VRARREDTRANS, ST.QTTOTAPRATST, 
    						 PR.SGUNIDADE, ST.CDDESTFILI, ST.CDCLIENTE, ST.QTCOMESTIST, ST.QTEFETCLIST, ST.QTCOMECLIST, 
    						 ST.QTPADRAOST, ST.QTCONTAIST, ST.QTTOTACONTST, ST.QTPARCPRATST, ST.QTCONTCOMPST, ST.VRPECONSST, 
    						 ST.QTAMOSTRAST, ST.QTTOTACONTSTCAMOS, ST.QTCONTCOMPSTCAMOS, ST.NRORG 
    				    FROM SERVTRAN ST, PRATO PR 
    				   WHERE (ST.CDFILIAL    = :P_CDFILIAL)   
    				     AND (ST.CDSERVFILI  = :P_CDSERVFILI) 
    				     AND (ST.DTSERVTRAN  = :P_DTSERVTRAN) 
    				     AND (ST.CDDESTFILI = :P_CDDESTFILI) 
    				     AND (ST.CDCLIENTE = :P_CDCLIENTE) 
    				     AND (ST.CDPRATO      = PR.CDPRATO)   
    				     AND (ST.CDFILAUXPRAT = PR.CDFILAUXPRAT)
    				   ORDER BY ST.CDPRATO, ST.CDFILAUXPRAT, ST.CDCLIENTE, ST.CDDESTFILI";
		$params = array(
			"P_CDFILIAL"   => $rows['CDFILIAL'],
			"P_CDSERVFILI" => $rows['CDSERVFILI'],
			"P_DTSERVTRAN" => $rows['DTSERVTRAN'],
			"P_CDDESTFILI" => $rows['CDDESTFILI'],
			"P_CDCLIENTE"  => $rows['CDCLIENTE']
		);

		$stmt = $this->connection->prepare($sqlPratos);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
    }
	
	public function updateFat(DTO\Request $request, DTO\Response $response) {
		try {
			$rowsUpdate = $request->getDataSet()->getRows();
			$cdoperador = $this->environment->getCdOperador();
	
			$sqlUpdate = "UPDATE SERVTRAN SET QTTOTAPRATST = :QTTOTAPRATST, QTPARCPRATST = :QTPARCPRATST, QTTOTACONTST = :QTTOTACONTST, QTCONTCOMPST = :QTCONTCOMPST, VRPECONSST = :VRPECONSST, QTPADRAOST = :QTPADRAOST, QTCONTAIST = :QTCONTAIST, QTEFETCLIST = :QTEFETCLIST, QTCOMESTIST = :QTCOMESTIST, QTCOMECLIST = :QTCOMECLIST, QTAMOSTRAST = :QTAMOSTRAST, QTTOTACONTSTCAMOS = :QTTOTACONTSTCAMOS, QTCONTCOMPSTCAMOS = :QTCONTCOMPSTCAMOS, DTULTATU = :DTULTATU, NRORGULTATU = :NRORGULTATU, CDOPERULTATU = :CDOPERULTATU WHERE CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND DTSERVTRAN = :OLD_DTSERVTRAN AND CDPRATO = :OLD_CDPRATO AND CDFILAUXPRAT = :OLD_CDFILAUXPRAT AND CDDESTFILI = :OLD_CDDESTFILI AND CDCLIENTE = :OLD_CDCLIENTE AND NRSERVTRAN = :OLD_NRSERVTRAN";
			             
            $this->connection->beginTransaction();    
			
			foreach($rowsUpdate as $rows) {
				$plates = $this->returnPlates($rows);
				$menu = $this->verifyMenu($rows);
				
				$rows['QTEFETCLIST'] = $this->valueToFloat($rows['QTEFETCLIST']);
	
				foreach ($plates as $row) {
					if (empty($menu) == true) {
						$comensais = $row['QTCOMESTIST'];
					}
					else {
						foreach ($menu as $plate) {
							if ($plate['CDPRATO'] == $row['CDPRATO']) {
								// $vrPeconsCard = $plate['VRPECONSCARD'];
								$comensais = $plate['QTCOMESTICAR'];
								break;
							}
						};	
					}
					
					if (isset($comensais) == false) $comensais = $rows['COMENSAIS'];
					
					$recipePlate = $this->verifyRecipePlate($row);
					$percentage = $this->verifyPercentage($recipePlate, $row);
					
					if (isset($percentage[0]) == false) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
					
					if ($row['QTEFETCLIST'] == 0 || $row['QTEFETCLIST'] == '') {
						if ($row['QTCOMECLIST'] > 0) $vrPeconsCard = 100 * $row['QTCOMESTIST'] / $row['QTCOMECLIST'];
						else throw new \Exception("Não é possível realizar a alteração pois os comensais estimados não possuem valor.");
					} else $vrPeconsCard = 100 * $row['QTCOMESTIST'] / $row['QTEFETCLIST'];
					
					if ($rows['QTEFETCLIST'] == 0 || $rows['QTEFETCLIST'] == '') $qtcomestist = $vrPeconsCard / 100 * $row['QTCOMECLIST'];
					else $qtcomestist = (($vrPeconsCard / 100) * $rows['QTEFETCLIST']);
	                
	                if ($rows['QTEFETCLIST'] == 0) $qttotapratst = round($row['QTPADRAOST'] * $qtcomestist, 5);
					else $qttotapratst = ($vrPeconsCard / 100 * $rows['QTEFETCLIST']) * $row['QTPADRAOST'];
	
					if ($percentage[0]['PERCENTUAL'] == NULL) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
					
	                $qtparcpratst = 0; /*QUANTIDADE DO CONTAINER INCOMPLETO*/
	                $qttotacontst = 0; /*TOTAL DE CONTAINERS*/
	                $qtcontcompst = 0; /*TOTAL DE CONTAINERS COMPLETOS*/
	                $qtcontincost = 0; /*TOTAL DE CONTAINERS INCOMPLETOS*/
					
					$qttotapratstcamos = 0;
			    	$qtparcpratstcamos = 0;
			    	$qttotacontstcamos = 0;
			    	$qtcontcompstcamos = 0;
			    	$qtcontincostcamos = 0;
			    	
			    	if ($row['QTPADRAOST'] != NULL || $row['QTPADRAOST'] != 0 && $row['QTCONTAIST'] != NULL || $row['QTCONTAIST'] != 0) {
				    	$qttotapratstcamos = $qttotapratst + $row['QTAMOSTRAST'];
						if ($qttotapratst >= $row['QTPADRAOST']) {
							$qtparcpratst = fmod($qttotapratstcamos, $row['QTCONTAIST']);
							$qttotacontst = intval($qttotapratst/$row['QTCONTAIST']);
							$qtcontcompst = $qttotacontst;
							
							if ($qtparcpratst > 0) {
								$qtcontincost = 1;
								$qttotacontst = $qttotacontst + 1;
								if ($percentage[0]['PERCENTUAL'] > 0) {
									if ($qtparcpratst < (($percentage[0]['PERCENTUAL'] / 100) * $row['QTCONTAIST'])) {
										$qtparcpratst = ($row['QTCONTAIST'] + $qtparcpratst) / 2;
										$qtcontincost = 2;
										// $qtcontcompst = $qtcontcompst - 1;
									}
								}
							}
						} else {
							$qtparcpratst = $qttotapratst;
						    $qttotacontst = 1;
						    $qtcontcompst = 0;
						    $qtcontincost = 1;
						}
	
				    	if ($qttotapratstcamos > $row['QTCONTAIST']) {
				    		$qtparcpratstcamos = fmod($qttotapratstcamos, $row['QTCONTAIST']);
				            $qttotacontstcamos = intval($qttotapratstcamos /$row['QTCONTAIST']);
				            $qtcontcompstcamos = $qttotacontstcamos;
				            
				            if ($qtparcpratstcamos > 0) {
				            	$qtcontincostcamos = 1;
				            	$qttotacontstcamos = $qttotacontstcamos + 1;
				            	
				            	if ($percentage[0]['PERCENTUAL'] > 0) {
				            		if ($qtparcpratstcamos - (($percentage[0]['PERCENTUAL'] / 100 ) * $row['QTCONTAIST']) < 0) {
				            			$qtparcpratstcamos = ($row['QTCONTAIST'] + $qtparcpratstcamos) / 2;
				                    	$qtcontincostcamos = 2;
				                    	$qtcontcompstcamos = $qtcontcompstcamos - 1;
				            		}
				            	}
				            }
				    	} else {
				    		$qtparcpratstcamos = $qttotapratstcamos;
				            $qttotacontstcamos = 1;
				            $qtcontcompstcamos = 0;
				            $qtcontincostcamos = 1;
				    	}
					}
					
					if ($qtcomestist == 0 || $qtcomestist == null) $vrpeconsst = 0;
					else $vrpeconsst = $qtcomestist * 100 / $comensais;
					
					$qtpadraost = $row['QTPADRAOST'];
					$qtcontaist = $row['QTCONTAIST'];
	
					$params = array(
						"OLD_CDFILIAL"      => $row['CDFILIAL'],
						"OLD_CDSERVFILI"    => $row['CDSERVFILI'],
						"OLD_DTSERVTRAN"    => $row['DTSERVTRAN'],
						"OLD_NRSERVTRAN"    => $row['NRSERVTRAN'],
						"OLD_CDDESTFILI"    => $row['CDDESTFILI'],
						"OLD_CDCLIENTE"     => $row['CDCLIENTE'],
						"OLD_CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
						"OLD_CDPRATO"       => $row['CDPRATO'],
						"QTTOTAPRATST"      => $qttotapratst,
						"QTPARCPRATST"      => $qtparcpratst,
						"QTTOTACONTST"      => $qttotacontst,
						"QTCONTCOMPST"      => $qtcontcompst,
						"VRPECONSST"        => $vrpeconsst,
						"QTPADRAOST"        => $qtpadraost,
						"QTCONTAIST"        => $qtcontaist,
						"QTCOMESTIST"       => $qtcomestist,
						"QTCOMECLIST"       => $row['QTCOMECLIST'],
						"QTEFETCLIST"       => $rows['QTEFETCLIST'],
						"QTAMOSTRAST"       => $row['QTAMOSTRAST'],
						"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
						"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
						"DTULTATU" => date("d/m/Y"),
						"NRORGULTATU" => $row['NRORG'],
						"CDOPERULTATU" => $cdoperador
					);
	
					$stmt = $this->connection->prepare($sqlUpdate);
					$stmt->execute($params);
				    $this->procedureEtiquetran($params, $qtcontcompstcamos, $qtcontincostcamos, $qtparcpratstcamos);
				}
			}
			
			$this->connection->commit();
		} catch (\Exception $e) {
			$this->connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
	}
	
	private function verifyMenu($rows){
		$sql = "SELECT
		             CO.CDFILIAL,
		             CO.NRCARDAPIO,
		             CO.CDFILAUXPRAT,
		             CO.CDPRATO,
		             SUM(CO.VRPECONSCARD) AS VRPECONSCARD,
		             CA.CDSERVFILI,
		             CA.DTCARDAPIO,
		             SUM(CA.QTCOMESTICAR) AS QTCOMESTICAR,
		             NVL(CA.CDDIETPAD,'XXXX') AS CDDIETPAD,
		             NVL(PT.VRARREDTRANS, 0) AS VRARREDTRANS,
		             PT.SGUNIDADE
		          FROM
		             COMPCARD CO,
		             CARDADIA CA,
		             PRATO PT
		          WHERE
		             (CA.CDFILIAL   = :P_CDFILIAL)   AND
		             (CA.CDSERVFILI = :P_CDSERVFILI) AND
		             (CA.DTCARDAPIO = :P_DTSERVTRAN) AND
		             (CO.CDFILIAL   = CA.CDFILIAL)   AND
		             (CO.NRCARDAPIO = CA.NRCARDAPIO) AND
		             (CO.CDPRATO    = PT.CDPRATO)    AND
		             (CO.CDFILAUXPRAT = PT.CDFILAUXPRAT)
				GROUP BY CO.CDFILIAL, CO.NRCARDAPIO, CO.CDFILAUXPRAT, CO.CDPRATO, CA.CDSERVFILI, CA.DTCARDAPIO, NVL(CA.CDDIETPAD,'XXXX'), NVL(PT.VRARREDTRANS, 0), PT.SGUNIDADE";
		
		$params = array(
			"P_CDFILIAL"   => $rows['CDFILIAL'],
			"P_CDSERVFILI" => $rows['CDSERVFILI'],
			"P_DTSERVTRAN" => $rows['DTSERVTRAN']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	private function verifyRecipePlate($row) {
		$sql = "SELECT 'RS' AS RECEITA FROM RECESERV WHERE CDFILIAL     = :P_CDFILIAL     AND CDSERVFILI   = :P_CDSERVFILI   AND CDPRATO      = :P_CDPRATO      AND CDFILAUXPRAT = :P_CDFILAUXPRAT UNION SELECT 'RL' AS RECEITA FROM RECEFILI RF WHERE RF.CDFILIAL     = :P_CDFILIAL     AND RF.CDPRATO      = :P_CDPRATO      AND RF.CDFILAUXPRAT = :P_CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL     = RF.CDFILIAL   AND RS.CDSERVFILI   = :P_CDSERVFILI AND RS.CDPRATO      = RF.CDPRATO    AND RS.CDFILAUXPRAT = RF.CDFILAUXPRAT) UNION SELECT 'RP' AS RECEITA FROM RECEPADR RP WHERE RP.CDPRATO      = :P_CDPRATO AND RP.CDFILAUXPRAT = :P_CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL     = :P_CDFILIAL   AND RS.CDSERVFILI   = :P_CDSERVFILI AND RS.CDPRATO      = RP.CDPRATO       AND RS.CDFILAUXPRAT = RP.CDFILAUXPRAT) AND NOT EXISTS (SELECT RF.CDPRATO FROM RECEFILI RF WHERE RF.CDFILIAL     = :P_CDFILIAL AND RF.CDPRATO      = RP.CDPRATO  AND RF.CDFILAUXPRAT = RP.CDFILAUXPRAT)";
		
		$params = array(
			"P_CDFILIAL"     => $row['CDFILIAL'],
			"P_CDSERVFILI"   => $row['CDSERVFILI'],
			"P_CDPRATO"      => $row['CDPRATO'],
			"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT']
		);

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	public function procedureEtiquetran($params, $qtcontincost, $qttotacontst, $qttotapratstcamos) {

		$procedure = new StoredProcedure($this->connection, 'GERA_ETIQUETRAN');
    	$procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $params['OLD_CDFILIAL'],     Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $params['OLD_CDSERVFILI'],   Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDDIETPADST',  Param::PARAM_INPUT, 'XXXX',                      Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDDESTFILI',   Param::PARAM_INPUT, $params['OLD_CDDESTFILI'],   Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_NRSERVTRAN',   Param::PARAM_INPUT, $params['OLD_NRSERVTRAN'],   Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDCLIENTE',    Param::PARAM_INPUT, $params['OLD_CDCLIENTE'],    Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_DTSERVTRAN',   Param::PARAM_INPUT, $params['OLD_DTSERVTRAN'],   Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDFILAUXPRAT', Param::PARAM_INPUT, $params['OLD_CDFILAUXPRAT'], Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_CDPRATO',      Param::PARAM_INPUT, $params['OLD_CDPRATO'],      Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTCOMPST', Param::PARAM_INPUT, $qtcontincost,               Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTAIST',   Param::PARAM_INPUT, $params['QTCONTAIST'],       Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTCONTINCOST', Param::PARAM_INPUT, $qttotacontst,               Param::PARAM_TYPE_STR));
    	$procedure->addParam(new Param('P_QTPARCPRATST', Param::PARAM_INPUT, $qttotapratstcamos,          Param::PARAM_TYPE_STR));
    	$procedure->execute();
	}
	
	private function verifyPercentage($recipePlate, $row) {
		if ($recipePlate[0]['RECEITA'] == 'RP') {
			$sql = "SELECT QTPADRAOPRAT AS QTPADRAO, QTCONTAIPRAT AS QTCONTAI, QTAMOSTRPADR AS QTAMOSTRAUN, VRPEMINPADR  AS PERCENTUAL FROM PRATPADR WHERE CDPRATO = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT";
			$params = array(
				"P_CDPRATO"      => $row['CDPRATO'],
				"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT']
			);
		} else if ($recipePlate[0]['RECEITA'] == 'RL') {
			$sql = "SELECT QTPADRAOFILI AS QTPADRAO, QTCONTAIFILI AS QTCONTAI, QTAMOSTRFILI AS QTAMOSTRAUN, VRPEMINFILI  AS PERCENTUAL FROM PRATFILI WHERE CDPRATO = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT AND CDFILIAL = :P_CDFILIAL";
			$params = array(
				"P_CDPRATO"      => $row['CDPRATO'],
				"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
				"P_CDFILIAL"     => $row['CDFILIAL'],
			);
		} else if ($recipePlate[0]['RECEITA'] == 'RS') {
			$sql = "SELECT QTPADRAOSERV AS QTPADRAO, QTCONTAISERV AS QTCONTAI, QTAMOSTRSERV AS QTAMOSTRAUN, VRPEMINSERV  AS PERCENTUAL FROM PRATSERV WHERE CDPRATO = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT AND CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI";
				$params = array(
				"P_CDPRATO"      => $row['CDPRATO'],
				"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
				"P_CDFILIAL"     => $row['CDFILIAL'],
				"P_CDSERVFILI"   => $row['CDSERVFILI']
			);
		}

		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $result = $stmt->fetchAll();
	    return $result;
	}
	
	private function verifyPercentageGeneral($unidade) {
		$sql = "SELECT NVL(VRPEMINCONT, '0') VRPEMINCONT FROM PARAMPLANEJA WHERE CDFILIAL = :P_CDFILIAL";
		
		$params = array(
			"P_CDFILIAL" => $unidade
		);
	    
	    $result = $this->connection->fetchColumn($sql, $params);
	}
	
	public function verifyPlateInsert(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			$validInsert = false;
			if ($row['WDGT'] == 'ADD') {
				$clientes = $this->verifyClient($response, $request);

				if (sizeof($clientes) == 0) $this->searchNewCodeInsert($row, $response, $request);
				else {
					foreach($clientes as $cliente) {
						if ($cliente['CDCLIENTE'] === $row['CDCLIENTE'] && $cliente['CDDESTFILI'] === $row['CDDESTFILI']) {
							$validInsert = true;
							throw new \Exception("Cliente e Ponto de Entrega já existem.");
						}
					}
					
					if ($validInsert != true) $this->searchNewCodeInsert($row, $response, $request);
				}
			} else {
				$sql = "SELECT (CDPRATO || CDFILAUXPRAT) AS CDCHAVEPRATO, CDPRATO, 
							CDFILAUXPRAT, CDFILIAL, CDSERVFILI, CDDIETPADST, DTSERVTRAN 
						  FROM SERVTRAN 
						 WHERE (CDPRATO      = :P_CDPRATO)      
						   AND (CDFILAUXPRAT = :P_CDFILAUXPRAT) AND (CDFILIAL     = :P_CDFILIAL)     
						   AND (CDSERVFILI   = :P_CDSERVFILI)   AND (CDCLIENTE    = :P_CDCLIENTE)    
						   AND (CDDESTFILI   = :P_CDDESTFILI)   AND (DTSERVTRAN   = :P_DTSERVTRAN)";
				
				$params = array(
					"P_CDPRATO"      => $row['CDPRATO'],
					"P_CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
					"P_CDFILIAL"     => $row['CDFILIAL'],
					"P_CDSERVFILI"   => $row['CDSERVFILI'],
					"P_CDCLIENTE"    => $row['CDCLIENTE'],
					"P_CDDESTFILI"   => $row['CDDESTFILI'],
					"P_DTSERVTRAN"   => $row['DTSERVTRAN']
				);
				
				$stmt = $this->connection->prepare($sql);
				$stmt->execute($params);
			    $result = $stmt->fetchAll();
			    if (sizeof($result) == 0) $this->searchNewCodeInsert($row, $response, $request);
			    else $response->addDataSet(new DataSet("result", $result));
			}
		} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }

	}
	
	private function searchNewCodeInsert($row, $response, $request) {
		$sql = "SELECT MAX(NRSERVTRAN) + 1 AS SEQUSERV 
		          FROM SERVTRAN 
		         WHERE (CDFILIAL    = :P_CDFILIAL)    AND (CDSERVFILI  = :P_CDSERVFILI)  
		           AND (DTSERVTRAN  = :P_DTSERVTRAN)  AND (CDDIETPADST = :P_CDDIETPADST)";
		
		$params = array(
			"P_CDFILIAL"    => $row['CDFILIAL'],
			"P_CDSERVFILI"  => $row['CDSERVFILI'],
			"P_DTSERVTRAN"  => $row['DTSERVTRAN'],
			"P_CDDIETPADST" => 'XXXX'
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	    $codeSeq = $stmt->fetchAll();
	    $this->insert($row, $codeSeq, $response, $request);
	}
	
	public function verifyClient($response, $request) {
		$requestRow = $request->getRow();
		$dataSourceName = 'clientesServtran';
		
		$conditions = array(
			array(
				"columnName" => "P_CDFILIAL",
				"operator" => "=",
				"value" => $requestRow['CDFILIAL']
			),
			array(
				"columnName" => "P_CDSERVFILI",
				"operator" => "=",
				"value" => $requestRow['CDSERVFILI']
			),
			array(
				"columnName" => "P_DTSERVTRAN",
				"operator" => "=",
				"value" => $requestRow['DTSERVTRAN']
			)
		);
		
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
	}
	
	private function insert($row, $codeSeq, $response, $request) {
		$sql = "INSERT INTO SERVTRAN 
					(CDFILIAL, CDSERVFILI, DTSERVTRAN, CDPRATO, CDFILAUXPRAT, CDDIETPADST, 
					 QTTOTAPRATST, CDDESTFILI, CDCLIENTE, QTCOMESTIST, QTCOMECLIST, QTPADRAOST,
					 QTCONTAIST, QTTOTACONTST, QTPARCPRATST, QTCONTCOMPST, VRPECONSST, NRSERVTRAN,
					 QTAMOSTRAST, QTTOTACONTSTCAMOS, QTCONTCOMPSTCAMOS, QTEFETCLIST) 
				VALUES (:OLD_CDFILIAL, :OLD_CDSERVFILI, :OLD_DTSERVTRAN, :OLD_CDPRATO, :OLD_CDFILAUXPRAT, :OLD_CDDIETPADST, 
						:QTTOTAPRATST, :OLD_CDDESTFILI, :OLD_CDCLIENTE, :QTCOMESTIST, :QTCOMECLIST, :QTPADRAOST, 
						:QTCONTAIST, :QTTOTACONTST, :QTPARCPRATST, :QTCONTCOMPST, :VRPECONSST, :OLD_NRSERVTRAN, 
						:QTAMOSTRAST, :QTTOTACONTSTCAMOS, :QTCONTCOMPSTCAMOS, :QTEFETCLIST)";
						
		$this->insertClientOrPlate($row, $sql, $codeSeq);
	}
	
	public function insertClientOrPlate($row, $sql, $codeSeq) {
		$row['QTCOMESTIST'] = $this->valueToFloat($row['QTCOMESTIST']);
		$row['QTCOMECLIST'] = $this->valueToFloat($row['QTCOMECLIST']);
		
		$comensais = $row['QTCOMECLIST'];
		
		$menu = $this->verifyMenu($row);
		
		$recipePlate = $this->verifyRecipePlate($row);
		$percentage = $this->verifyPercentage($recipePlate, $row);
		
		if (isset($percentage[0]) == false) {
			$percentage[0]['QTPADRAO'] = $row['QTPADRAOST'];
			$percentage[0]['QTCONTAI'] = $row['QTCONTAIST'];
			$percentage[0]['QTAMOSTRAUN'] = $row['QTAMOSTRAST'];
			$percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
		}
		
		if ($percentage[0]['QTPADRAO'] != null || $percentage[0]['QTPADRAO'] != 0) $row['QTPADRAOST'] = $percentage[0]['QTPADRAO'];
		
		if ($percentage[0]['QTCONTAI'] != null || $percentage[0]['QTCONTAI'] != 0) $row['QTCONTAIST'] = $percentage[0]['QTCONTAI'];
		
		if ($percentage[0]['QTAMOSTRAUN'] != null || $percentage[0]['QTAMOSTRAUN'] != 0) $row['QTAMOSTRAST'] = $percentage[0]['QTAMOSTRAUN'];
		
		if ($percentage[0]['PERCENTUAL'] == NULL) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
		
		foreach ($menu as $plate) {
			if ($plate['CDPRATO'] == $row['CDPRATO']) {
				$vrPeconsCard = $plate['VRPECONSCARD'];
				$comensais = $plate['QTCOMESTICAR'];
				break;
			}
		};
		
		// if (isset($vrPeconsCard) == false) $vrPeconsCard = 100;
		// if (isset($vrPeconsCard) == false) $vrPeconsCard = 100 * $row['QTCOMESTIST'] / $row['QTCOMECLIST'];
		if ($row['QTEFETCLIST'] == 0 || $row['QTEFETCLIST'] == '') $vrPeconsCard = 100 * $row['QTCOMESTIST'] / $row['QTCOMECLIST'];
		else $vrPeconsCard = (100 * ($row['QTCOMESTIST'] / $row['QTEFETCLIST']));

		$row['VRARREDTRANS'] = $vrPeconsCard;
		
		$row['QTCOMESTIST'] = $this->valueToFloat($row['QTCOMESTIST']);
		$row['QTCOMECLIST'] = $this->valueToFloat($row['QTCOMECLIST']);
		$row['QTAMOSTRAST'] = $this->valueToFloat($row['QTAMOSTRAST']);
		$row['QTCONTAIST'] = $this->valueToFloat($row['QTCONTAIST']);
		$row['QTPADRAOST'] = $this->valueToFloat($row['QTPADRAOST']);
		
		if (is_int($row['QTCOMESTIST']) == true) $row['QTCOMESTIST'] = floatval($row['QTCOMESTIST']);
		if (is_string($row['QTPADRAOST']) == true) $row['QTPADRAOST'] = floatval($row['QTPADRAOST']);
		// $qtcomestist = $row['QTCOMESTIST'];
		
		if ($row['QTEFETCLIST'] == 0 || $row['QTEFETCLIST'] == '') $qtcomestist = (($vrPeconsCard / 100) * $row['QTCOMECLIST']);
		else $qtcomestist = (($vrPeconsCard/100) * $row['QTEFETCLIST']);
		
		$qttotapratst = round($row['QTPADRAOST'] * $row['QTCOMESTIST'], 3);
		
		$row['QTTOTACONTST'] = ceil((($qtcomestist * $row['QTPADRAOST']) / $row['QTCONTAIST']));
		$row['QTCONTCOMPST'] = intval((($qtcomestist * $row['QTPADRAOST']) / $row['QTCONTAIST']), 2);

		if ($row['QTPADRAOST'] == NULL || $row['QTPADRAOST'] == 0 && $row['QTCONTAIST'] == NULL || $row['QTCONTAIST'] == 0) {
		  	if ($row['VRARREDTRANS'] != 0) {
		  		if ($qttotapratst > 0 && fmod($qttotapratst, $row['VRARREDTRANS']) > 0) 
		  			$qttotapratst = (intval($qttotapratst/$row['VRARREDTRANS']) + 1) * $row['VRARREDTRANS'];
		  	}
		}
		
		$code = str_pad($codeSeq[0]['SEQUSERV'] + 1 , 10, '0' , STR_PAD_LEFT);
		
	    $qtparcpratst = 0; /*QUANTIDADE DO CONTAINER INCOMPLETO*/
        $qttotacontst = 0; /*TOTAL DE CONTAINERS*/
        $qtcontcompst = 0; /*TOTAL DE CONTAINERS COMPLETOS*/
        $qtcontincost = 0; /*TOTAL DE CONTAINERS INCOMPLETOS*/
		
		$qttotapratstcamos = 0;
    	$qtparcpratstcamos = 0;
    	$qttotacontstcamos = 0;
    	$qtcontcompstcamos = 0;
    	$qtcontincostcamos = 0;
    	
		if (($row['QTCONTAIST'] != NULL || $row['QTCONTAIST'] != 0) && ($row['QTPADRAOST'] != NULL || $row['QTPADRAOST'] != 0)) {
	    	$qttotapratstcamos = $qttotapratst + $row['QTAMOSTRAST'];
	    	
			if ($qttotapratst >= $row['QTCONTAIST']) {
				$qtparcpratst = fmod($qttotapratstcamos, $row['QTCONTAIST']);
				$qttotacontst = intval($qttotapratst/$row['QTCONTAIST']);
				$qtcontcompst = $qttotacontst;
				
				if ($qtparcpratst > 0) {
					$qtcontincost = 1;
					$qttotacontst = $qttotacontst + 1;
					if ($percentage[0]['PERCENTUAL'] > 0) {
						if ($qtparcpratst < (($percentage[0]['PERCENTUAL']/100) * $row['QTCONTAIST'])) {
							$qtparcpratst = ($row['QTCONTAIST'] + $qtparcpratst) / 2;
							$qtcontincost = 2;
							$qtcontcompst = $qtcontcompst - 1;
						}
					}
				}
			} else {
				$qtparcpratst = $qttotapratst;
			    $qttotacontst = 1;
			    $qtcontcompst = 0;
			    $qtcontincost = 1;
			}
	    	

	    	if ($qttotapratstcamos > $row['QTCONTAIST']) {
	    		$qtparcpratstcamos = fmod($qttotapratstcamos, $row['QTCONTAIST']);
	            $qttotacontstcamos = intval($qttotapratstcamos / $row['QTCONTAIST']);
	            $qtcontcompstcamos = $qttotacontstcamos;
	            if ($qtparcpratstcamos > 0) {
	            	$qtcontincostcamos = 1;
	            	$qttotacontstcamos = $qttotacontstcamos + 1;
	            	
	            	if ($percentage[0]['PERCENTUAL'] > 0) {
	            		if ($qtparcpratstcamos < (($percentage[0]['PERCENTUAL'] / 100 ) * $row['QTCONTAIST'])) {
	            			$qtparcpratstcamos = ($row['QTCONTAIST'] + $qtparcpratstcamos) / 2;
	                    	$qtcontincostcamos = 2;
	                    	$qtcontcompstcamos = $qtcontcompstcamos - 1;
	            		}
	            	}
	            }
	    	} else {
	    		$qtparcpratstcamos = $qttotapratstcamos;
	            $qttotacontstcamos = 1;
	            $qtcontcompstcamos = 0;
	            $qtcontincostcamos = 1;
	    	}
		} 
		
		if ($qtcomestist == 0 || $qtcomestist == null) $vrpeconsst = 0;
		else $vrpeconsst = $qtcomestist * 100 / $row['QTCOMESTIST'];
	
		$params = array(
			"OLD_CDFILIAL"      => $row['CDFILIAL'],
			"OLD_CDSERVFILI"    => $row['CDSERVFILI'],
			"OLD_DTSERVTRAN"    => $row['DTSERVTRAN'],
			"OLD_CDPRATO"       => $row['CDPRATO'],
			"OLD_CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
			"OLD_CDDIETPADST"   => 'XXXX',
			"QTTOTAPRATST"      => $qttotapratst,
			"OLD_CDDESTFILI"    => $row['CDDESTFILI'],
			"OLD_CDCLIENTE"     => $row['CDCLIENTE'],
			"QTCOMESTIST"       => $qtcomestist,
			"QTCOMECLIST"       => $row['QTCOMECLIST'],
			"QTPADRAOST"        => $row['QTPADRAOST'],
			"QTCONTAIST"        => $row['QTCONTAIST'],
			"QTTOTACONTST"      => $row['QTTOTACONTST'],
			"QTPARCPRATST"      => round($qtparcpratst, 20),
			"QTCONTCOMPST"      => $row['QTCONTCOMPST'],
			"VRPECONSST"        => $vrpeconsst,
			"OLD_NRSERVTRAN"    => $code,
			"QTAMOSTRAST"       => $row['QTAMOSTRAST'],
			"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
			"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
			"QTEFETCLIST"       => round($row['QTEFETCLIST'], 20)
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->procedureEtiquetran($params, $qtcontcompstcamos, $qtcontincost, $qtparcpratstcamos);
	}
	
	public function deleteEtiqueta(DTO\Request $request, DTO\Response $response) {
		$row = $request->getDataSet()->getRows();
		$sql = "DELETE FROM ETIQUETRAN 
		         WHERE ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI )
		           AND ( CDDESTFILI = :CDDESTFILI ) AND ( CDCLIENTE = :CDCLIENTE ) 
		           AND ( DTSERVTRAN = :DTSERVTRAN ) AND ( DTSERVTRAN = :DTSERVTRAN )";
		
		$params = array(
			"CDFILIAL"   => $row[0]['CDFILIAL'],
			"CDSERVFILI" => $row[0]['CDSERVFILI'],
			"CDDESTFILI" => $row[0]['CDDESTFILI'],
			"CDCLIENTE"  => $row[0]['CDCLIENTE'],
			"DTSERVTRAN" => $row[0]['DTSERVTRAN']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->deleteServtran($row);
	}
	
	private function deleteServtran($row) {
		$sql = "DELETE FROM SERVTRAN 
				 WHERE (CDFILIAL     = :P_CDFILIAL) AND (CDSERVFILI   = :P_CDSERVFILI) 
				   AND (DTSERVTRAN   = :P_DTSERVTRAN) AND (CDDESTFILI   = :P_CDDESTFILI) 
				   AND (CDCLIENTE    = :P_CDCLIENTE)";
		
		$params = array(
			"P_CDFILIAL"   => $row[0]['CDFILIAL'],
			"P_CDSERVFILI" => $row[0]['CDSERVFILI'],
			"P_DTSERVTRAN" => $row[0]['DTSERVTRAN'],
			"P_CDDESTFILI" => $row[0]['CDDESTFILI'],
			"P_CDCLIENTE"  => $row[0]['CDCLIENTE']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	}
	
	public function updatePlate(DTO\Request $request, DTO\Response $response) {
		try {
			$rowsUpdate = $request->getDataSet()->getRows();
			$cdoperador = $this->environment->getCdOperador();
			
			$sqlUpdate = "UPDATE SERVTRAN SET QTTOTAPRATST = :QTTOTAPRATST, QTPARCPRATST = :QTPARCPRATST, QTTOTACONTST = :QTTOTACONTST, QTCONTCOMPST = :QTCONTCOMPST, VRPECONSST = :VRPECONSST, QTPADRAOST = :QTPADRAOST, QTCONTAIST = :QTCONTAIST, QTEFETCLIST = :QTEFETCLIST, QTCOMESTIST = :QTCOMESTIST, QTCOMECLIST = :QTCOMECLIST, QTAMOSTRAST = :QTAMOSTRAST, QTTOTACONTSTCAMOS = :QTTOTACONTSTCAMOS, QTCONTCOMPSTCAMOS = :QTCONTCOMPSTCAMOS, DTULTATU = :DTULTATU, NRORGULTATU = :NRORGULTATU, CDOPERULTATU = :CDOPERULTATU WHERE CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND DTSERVTRAN = :OLD_DTSERVTRAN AND CDPRATO = :OLD_CDPRATO AND CDFILAUXPRAT = :OLD_CDFILAUXPRAT AND CDDESTFILI = :OLD_CDDESTFILI AND CDCLIENTE = :OLD_CDCLIENTE AND NRSERVTRAN = :OLD_NRSERVTRAN";
			
			$this->connection->beginTransaction();  
			
			foreach ($rowsUpdate as $row) {
				$menu = $this->verifyMenu($row);
				
				$recipePlate = $this->verifyRecipePlate($row);
				$percentage = $this->verifyPercentage($recipePlate, $row);
				
				if (isset($percentage[0]) == false) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
				
				if ($percentage[0]['PERCENTUAL'] == NULL) $percentage[0]['PERCENTUAL'] = $this->verifyPercentageGeneral($row['CDFILIAL']);
		
				foreach ($menu as $plate){
					if ($plate['CDPRATO'] == $row['CDPRATO']) {
						// $vrPeconsCard = $plate['VRPECONSCARD'];
						$comensais = $plate['QTCOMESTICAR'];
						break;
					}
				};
				
				if(isset($comensais) == false) $comensais = $row['QTCOMECLIST'];
				$row['COMENSAIS'] = $this->valueToFloat($row['COMENSAIS']);
				$row['QTPADRAOST'] = $this->valueToFloat($row['QTPADRAOST']);
				
				if ($row['COMENSAIS'] == 0 || $row['COMENSAIS'] == null) $vrpeconsst = 0;
				else $vrpeconsst = $row['COMENSAIS'] * 100 / $comensais;
				
				if ($row['QTEFETCLIST'] == 0 || $row['QTEFETCLIST'] == '') {
					if ($row['QTCOMECLIST'] > 0) $vrPeconsCard = 100 * $row['COMENSAIS'] / $row['QTCOMECLIST'];
					else $vrPeconsCard = 100;
				} else $vrPeconsCard = (100 * ($row['COMENSAIS'] / $row['QTEFETCLIST']));
				
				if ($row['QTEFETCLIST'] == 0 || $row['QTEFETCLIST'] == '') {
					if ($row['QTCOMECLIST'] > 0) $qtcomestist = (($vrPeconsCard / 100) * $row['QTCOMECLIST']);
					else $qtcomestist = $row['COMENSAIS'];
				} else $qtcomestist = (($vrPeconsCard / 100) * $row['QTEFETCLIST']);
				
				$qttotapratst = round($row['COMENSAIS'] * floatval($row['QTPADRAOST']), 3);
				$qtcontcompst = ($qttotapratst + $row['QTAMOSTRAST'])/$row['QTCONTAIST'];
				
	            $qtparcpratst = 0; /*QUANTIDADE DO CONTAINER INCOMPLETO*/
	            $qttotacontst = 0; /*TOTAL DE CONTAINERS*/
	            $qtcontcompst = 0; /*TOTAL DE CONTAINERS COMPLETOS*/
	            $qtcontincost = 0; /*TOTAL DE CONTAINERS INCOMPLETOS*/
				$qttotapratstcamos = 0;
		    	$qtparcpratstcamos = 0;
		    	$qttotacontstcamos = 0;
		    	$qtcontcompstcamos = 0;
		    	$qtcontincostcamos = 0;
	
				if (($row['QTCONTAIST'] != NULL || $row['QTCONTAIST'] != 0) && ($row['QTPADRAOST'] != NULL || $row['QTPADRAOST'] != 0)) {
			    	$qttotapratstcamos = $qttotapratst + $row['QTAMOSTRAST'];
					if ($qttotapratstcamos >= $row['QTCONTAIST']) {
		 				$qtparcpratst = fmod($qttotapratstcamos, $row['QTCONTAIST']);
						$qttotacontst = intval($qttotapratst/$row['QTCONTAIST']);
						$qtcontcompst = $qttotacontst;
	
						if ($qtparcpratst > 0) {
							$qtcontincost = 1;
							$qttotacontst = $qttotacontst + 1;
							if ($percentage[0]['PERCENTUAL'] > 0) {
								if ($qtparcpratst < (($percentage[0]['PERCENTUAL']/100) * $row['QTCONTAIST'])) {
									$qtparcpratst = ($row['QTCONTAIST'] + $qtparcpratst) / 2;
									$qtcontincost = 2;
									$qtcontcompst = $qtcontcompst - 1;
								}
							}
						}
					} else {
						$qtparcpratst = $qttotapratst;
					    $qttotacontst = 1;
					    $qtcontcompst = 0;
					    $qtcontincost = 1;
					}
					
			    	if ($qttotapratstcamos > $row['QTCONTAIST']) {
			    		$qtparcpratstcamos = fmod($qttotapratstcamos, $row['QTCONTAIST']);
			            $qttotacontstcamos = intval($qttotapratstcamos / $row['QTCONTAIST']);
			            $qtcontcompstcamos = $qttotacontstcamos; ///*QUANTIDADE DE CONTAINERS COMPLETOS com amostras*/
			            
			            if ($qtparcpratstcamos > 0) { // num total de embalagens
			            	$qtcontincostcamos = 1; // qtd container incompletos com amostra
			            	$qttotacontstcamos = $qttotacontstcamos + 1; //TOTAL DE CONTAINERS com amostras
			            	if ($percentage[0]['PERCENTUAL'] > 0) {
			            		if ($qtparcpratstcamos < (($percentage[0]['PERCENTUAL'] / 100 ) * $row['QTCONTAIST'])) {
			            			$qtparcpratstcamos = ($row['QTCONTAIST'] + $qtparcpratstcamos) / 2;
			                    	$qtcontincostcamos = 2;
			                    	$qtcontcompstcamos = $qtcontcompstcamos - 1;
			            		}
			            	}
			            }
			    	} else {
			    		$qtparcpratstcamos = $qttotapratstcamos;
			            $qttotacontstcamos = 1;
			            $qtcontcompstcamos = 0;
			            $qtcontincostcamos = 1;
			    	}
				} 
				$params = array(
					"OLD_CDFILIAL"      => $row['CDFILIAL'],
					"OLD_CDSERVFILI"    => $row['CDSERVFILI'],
					"OLD_DTSERVTRAN"    => $row['DTSERVTRAN'],
					"OLD_NRSERVTRAN"    => $row['NRSERVTRAN'],
					"OLD_CDDESTFILI"    => $row['CDDESTFILI'],
					"OLD_CDCLIENTE"     => $row['CDCLIENTE'],
					"OLD_CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
					"OLD_CDPRATO"       => $row['CDPRATO'],
					"QTTOTAPRATST"      => round($qttotapratst, 3),
					"QTPARCPRATST"      => $qtparcpratst,
					"QTTOTACONTST"      => $qttotacontst,
					"QTCONTCOMPST"      => $qtcontcompst,
					"VRPECONSST"        => $vrpeconsst,
					"QTPADRAOST"        => $row['QTPADRAOST'], 
					"QTCONTAIST"        => $row['QTCONTAIST'],
					"QTCOMESTIST"       => $qtcomestist,
					"QTCOMECLIST"       => $row['QTCOMECLIST'],
					"QTEFETCLIST"       => $row['QTEFETCLIST'],
					"QTAMOSTRAST"       => $this->valueToFloat($row['QTAMOSTRAST']),
					"QTTOTACONTSTCAMOS" => ceil($qttotacontstcamos),
					"QTCONTCOMPSTCAMOS" => intval($qtcontcompstcamos),
					"DTULTATU"          => date("d/m/Y"),
					"NRORGULTATU"       => $row['NRORG'],
					"CDOPERULTATU"      => $cdoperador
				);
				
				$stmt = $this->connection->prepare($sqlUpdate);
				$stmt->execute($params);
			    $this->procedureEtiquetran($params, $qtcontcompstcamos, $qtcontincost, $qtparcpratstcamos);
			} 
			
			$this->connection->commit();
		} catch (\Exception $e) {
			$this->connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
	}
	
	public function deletePlate(DTO\Request $request, DTO\Response $response) {
		$row = $request->getDataSet()->getRows();
		$sql = "DELETE FROM SERVTRAN 
				 WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDSERVFILI = :P_CDSERVFILI ) 
				   AND ( DTSERVTRAN = :P_DTSERVTRAN ) AND ( CDDESTFILI = :P_CDDESTFILI ) 
				   AND ( CDCLIENTE = :P_CDCLIENTE ) AND ( NRSERVTRAN = :P_NRSERVTRAN )";

		$params = array(
			"P_CDFILIAL"   => $row[0]['CDFILIAL'],
			"P_CDSERVFILI" => $row[0]['CDSERVFILI'],
			"P_DTSERVTRAN" => $row[0]['DTSERVTRAN'],
			"P_CDDESTFILI" => $row[0]['CDDESTFILI'],
			"P_CDCLIENTE"  => $row[0]['CDCLIENTE'],
			"P_NRSERVTRAN" => $row[0]['NRSERVTRAN']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	}
	
	function valueToFloat($value){
        if(is_string($value) && !$this->isEmptyOrBlank($value)){
            $dotPos   = strpos($value, ".");
            $commaPos = strpos($value, ",");
            if($commaPos == false && $dotPos == false){
                return floatval($value);
            } 
            $dotPos   = $dotPos   !== false ? $dotPos   : -1;
            $commaPos = $commaPos !== false ? $commaPos : -1;
            if($commaPos > $dotPos){
                $firstReplace = str_replace(".", "", $value);
                $secondReplace = str_replace(",", ".", $firstReplace);
                return floatval($secondReplace);
            } 
            if($commaPos < $dotPos){
                $firstReplace = str_replace(",", "", $value);
                return floatval($firstReplace);
            }
        } else {
            return $value;
        }
    }

    function isEmptyOrBlank($value){
        return !isset($value) || $this->isBlank($value);
    }
    
    function isBlank($value){
        $newStr = str_replace(" ", "", $value);
        return strlen($newStr) === 0;
    }
}